<?php

declare(strict_types=1);

namespace Drupal\posthog_php;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use PostHog\PostHog;

/**
 * Mirrors the PostHog class from PostHog SDK.
 */
class Sdk implements SdkInterface {

  /**
   * Constructs a Sdk object.
   */
  public function __construct(
    readonly ConfigFactoryInterface $configFactory,
    readonly LoggerChannelFactoryInterface $loggerChannelFactory,
    readonly ModuleHandlerInterface $moduleHandler,
    protected $initalized = FALSE,
  ) {
    $this->init();
  }

  /**
   * {@inheritDoc}
   */
  public static function getVersion(): string {
    return PostHog::VERSION;
  }

  /**
   * {@inheritdoc}
   */
  public function init(): void {
    if ($this->initalized) {
      return;
    }
    $config = $this->configFactory->get('posthog.settings');
    $apiKey = $config->get('api_key');
    $host = $config->get('host');
    PostHog::init($apiKey,
      ['host' => $host]
    );
    $this->initalized = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function identify($distinctId, array $properties = []): bool {
    if (!$this->initalized) {
      $this->init();
    }
    $message = [
      'distinctId' => $distinctId,
    ];
    if (!empty($properties)) {
      $capture['properties'] = $properties;
    }

    $success = PostHog::identify($message);

    if (!$success && (bool) $this->configFactory->get('posthog.settings')->get('enable_logging')) {
      $this->loggerChannelFactory->get('posthog_php')->error('Failed to identify user with distinct id: %did.', [
        '%did' => $distinctId,
      ]);
    }
    return $success;
  }

  /**
   * {@inheritdoc}
   */
  public function groupIdentify(string $groupType, mixed $groupKey, array $properties = []): bool {
    if (!$this->initalized) {
      $this->init();
    }
    $message = [
      'groupType' => $groupType,
      'groupKey' => $groupKey,
    ];
    if (!empty($properties)) {
      $capture['properties'] = $properties;
    }

    $success = PostHog::groupIdentify($message);
    if (!$success && (bool) $this->configFactory->get('posthog.settings')->get('enable_logging')) {
      $this->loggerChannelFactory->get('posthog_php')->error('Failed to group identify group with group key: %gKey.', [
        '%gKey' => $groupKey,
      ]);
    }
    return $success;
  }

  /**
   * {@inheritdoc}
   */
  public function isFeatureEnabled(string $key, string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE, bool $sendFeatureFlagEvents = TRUE): ?bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::isFeatureEnabled($key, $distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally, $sendFeatureFlagEvents);
  }

  /**
   * {@inheritdoc}
   */
  public function getFeatureFlag(string $key, string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE, bool $sendFeatureFlagEvents = TRUE): null|bool|string {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::getFeatureFlag($key, $distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally, $sendFeatureFlagEvents);
  }

  /**
   * {@inheritdoc}
   */
  public function getAllFlags(string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE): array {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::getAllFlags($distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally);
  }

  /**
   * {@inheritdoc}
   */
  public function fetchFeatureVariants(string $distinctId, array $groups = []): array {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::fetchFeatureVariants($distinctId, $groups);
  }

  /**
   * {@inheritdoc}
   */
  public function alias(mixed $distinctId, mixed $alias): bool {
    if (!$this->initalized) {
      $this->init();
    }
    $message = [
      'distinctId' => $distinctId,
      'alias' => $alias,
    ];
    $success = PostHog::alias($message);

    if (!$success && (bool) $this->configFactory->get('posthog.settings')->get('enable_logging')) {
      $this->loggerChannelFactory->get('posthog_php')->error('Failed to alias user with distinct id: %did.', [
        '%did' => $distinctId,
      ]);
    }

    return $success;
  }

  /**
   * {@inheritdoc}
   */
  public function raw(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::raw($message);
  }

  /**
   * {@inheritdoc}
   */
  public function validate($msg, $type): void {
    if (!$this->initalized) {
      $this->init();
    }
    PostHog::validate($msg, $type);
  }

  /**
   * {@inheritdoc}
   */
  public function flush(): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::flush();
  }

  /**
   * {@inheritDoc}
   */
  public function capture(array $message, array $context = []): bool {
    if (!$this->initalized) {
      $this->init();
    }
    $config = $this->configFactory->get('posthog_php.settings');

    // If the distinct id is NULL, we want to generate our own id:
    if ($message['distinctId'] === NULL) {
      $message['distinctId'] = uniqid('anonymous-');
    }

    // IF the distinct id is NULL, or the process_person_profile is disabled,
    // we won't process person profiles and therefore handle this as an
    // anonymous event:
    if (!$config->get('process_person_profile') || $message['distinctId'] === NULL) {
      $message['properties']['$process_person_profile'] = FALSE;
    }

    $this->moduleHandler->alter('posthog_capture', $message, $context);
    $success = PostHog::capture($message);

    if (!$success && (bool) $config->get('enable_logging')) {
      $this->loggerChannelFactory->get('posthog_php')->error('Failed to capture user event for user with distinct id: %did.', [
        '%did' => $message['distinctId'],
      ]);
    }

    return $success;
  }

  /**
   * {@inheritDoc}
   */
  public function captureUserEvent(?string $distinctId, string $event, array $properties = [], array $context = []): bool {
    $capture = [
      'distinctId' => $distinctId,
      'event' => $event,
    ];

    if (!empty($properties)) {
      $capture['properties'] = $properties;
    }

    return $this->capture($capture, $context);
  }

  /**
   * {@inheritDoc}
   */
  public function captureUserPageView(?string $distinctId, string $currentUrl, array $context = []): bool {
    return $this->captureUserEvent($distinctId, SdkInterface::CAPTURE_EVENT_PAGEVIEW, ['$current_url' => $currentUrl], $context);
  }

}
