<?php

declare(strict_types=1);

namespace Drupal\posthog;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides posthog uesr properties.
 */
final class UserAttributesProvider {

  /**
   * The posthog settings configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $posthogSettings;

  /**
   * Constructs a PropertyProvider object.
   */
  public function __construct(
    private readonly ConfigFactoryInterface $configFactory,
    private readonly AccountInterface $currentUser,
  ) {
    $this->posthogSettings = $this->configFactory->get('posthog.settings');
  }

  /**
   * Returns a user's distinct id based on the configuration.
   */
  public function getUserDistinctId(AccountInterface $user): ?string {
    // Anonymous users, can not be identified by their user id, email or name,
    // so we simply return NULL:
    if ($user->isAnonymous()) {
      return NULL;
    }
    switch ($this->posthogSettings->get('distinct_id')) {
      case 'uid':
        return (string) $user->id();

      case 'mail':
        return $user->getEmail();

      case 'name':
        return $user->getAccountName();

      default:
      case 'no_identification':
        return NULL;
    }
  }

  /**
   * Returns the current user's distinct id based on the configuration.
   */
  public function getCurrentUserDistinctId(): ?string {
    return $this->getUserDistinctId($this->currentUser);
  }

  /**
   * Retrieves user properties.
   *
   * @return array
   *   An associative array of user properties.
   */
  public function getUserProperties(AccountInterface $user): array {
    $userProperties = [];

    // Get the user properties from the configuration and set them inside the
    // user properties array:
    $userSettingsProperties = $this->posthogSettings->get('user_properties');

    if (!empty($userSettingsProperties['uid'])) {
      $userProperties['uid'] = $user->id();
    }
    if (!empty($userSettingsProperties['email'])) {
      $userProperties['email'] = $user->getEmail();
    }
    if (!empty($userSettingsProperties['name'])) {
      $userProperties['name'] = $user->getAccountName();
    }

    return $userProperties;
  }

  /**
   * Retrieves current user properties.
   */
  public function getCurrentUserProperties(): array {
    return $this->getUserProperties($this->currentUser);
  }

}
